<?php

namespace Drupal\botlog;

use Drupal\Core\Config\ConfigFactory;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides helper methods for botlog module.
 */
class BotlogHelper {

  /**
   * The warning status level.
   *
   * @var int
   */
  public const STATUS_WARN = 1;

  /**
   * The warning status level.
   *
   * @var int
   */
  public const STATUS_ERROR = 2;

  /**
   * The module name.
   *
   * @var string
   */
  protected const MODULE = 'botlog';

  /**
   * The configuration manager.
   *
   * @var Drupal\Core\Config\ConfigFactory
   */
  protected $cfg;

  /**
   * The current request.
   *
   * @var Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The SQL.
   *
   * @var Drupal\botlog\BotlogSql
   */
  protected $sql;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Config\ConfigFactory $cfg
   *   The configuration manager.
   * @param Drupal\botlog\BotlogSql $sql
   *   The current request.
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   */
  public function __construct(ConfigFactory $cfg, BotlogSql $sql, RequestStack $request) {
    $this->cfg = $cfg;
    $this->request = $request->getCurrentRequest();
    $this->sql = $sql;
  }

  /**
   * Delete all entries with the supplied ID.
   *
   * @param string $ip
   *   The IP.
   *
   * @return int
   *   The Number of rows deleted.
   */
  public function deleteByIp($ip) {
    return $this->sql->deleteByIp($ip);
  }

  /**
   * Delete all entries older than the UNIX timestamp.
   *
   * @param string $timestamp
   *   The UNIX timestamp.
   *
   * @return int
   *   The Number of rows deleted.
   */
  public function deleteByTime($timestamp) {
    return $this->sql->deleteByTime($timestamp);
  }

  /**
   * Insert an IP with error status.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function errorIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_ERROR,
      'data'      => print_r($data, TRUE),
    ];
    return $this->sql->insertIp($fields);
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   * @param bool $reset
   *   Flush config cache.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function getCfg($key = '', $reset = FALSE) {
    $id = $this->getModuleName() . '.config';

    if ($reset) {
      $this->cfg->reset($id);
    }
    $cfg = $this->cfg->get($id);

    return $key ? $cfg->get($key) : (object) $cfg->get();
  }

  /**
   * Fetch the row count.
   *
   * @return int
   *   The number of rows in the botlog table.
   */
  public function getCount() {
    return $this->sql->getCount();
  }

  /**
   * Fetch the row count.
   *
   * @param int $ip
   *   The IP.
   *
   * @return int
   *   The number of rows in the botlog table for the IP.
   */
  public function getIpCount($ip) {
    return $this->sql->getIpCount($ip);
  }

  /**
   * Get the status of the IP.
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   The block/no-block status.
   */
  public function getErrorStatus($ip) {
    return $this->sql->getErrorStatus($ip, static::STATUS_ERROR) ? static::STATUS_ERROR : 0;
  }

  /**
   * Public access to the FACETS.
   *
   * @return array
   *   The FACETS.
   */
  public function getFacets() {
    return !empty($this->getCfg('facets')) ? (array) $this->getCfg('facets') : [];
  }

  /**
   * Get the field target ID.
   *
   * @param object $view
   *   The view object.
   *
   * @return array
   *   An array of input args.
   */
  public function getInput($view) {
    $data = [];
    $facets = $view->getExposedInput();

    if (empty($facets['f'])) {
      return [];
    }
    foreach ($facets['f'] as $arg) {
      if (!preg_match('/^(\w+):(\w+)$/', $arg, $m)) {
        continue;
      }
      $data[$m[1]][] = $m[2];
    }
    return !empty($data) ? $data : [];
  }

  /**
   * Public access to the FACETS.
   *
   * @return array
   *   The FACETS.
   */
  public function getIp() {
    return $this->request->getClientIp();
  }

  /**
   * Get the module's name.
   *
   * @return string
   *   The module's name.
   */
  public function getModuleName() {
    return static::MODULE;
  }

  /**
   * Insert an IP with warning status.
   *
   * @param string $field
   *   The field name.
   * @param int $limit
   *   The set limt of occurances.
   * @param int $facets
   *   The number of facets.
   *
   * @return string
   *   Message text.
   */
  public function getMsg($field, $limit, $facets) {
    return "Error:\n$field limit: $limit\n$field count: $facets\n\n";
  }

  /**
   * Get the status of the IP.
   *
   * @param string $ip
   *   The IP.
   *
   * @return bool
   *   The block/no-block status.
   */
  public function getStatus($ip) {
    return $this->getErrorStatus($ip);
  }

  /**
   * Get the field target ID.
   *
   * @param object $obj
   *   The node object.
   * @param string $field
   *   The field name.
   *
   * @return int
   *   The field ID.
   */
  public function getTgt($obj, $field) {
    return (!empty($obj) && !empty($field) && $obj->hasField($field) && !$obj->get($field)->isEmpty()) ? $obj->get($field)->target_id : 0;
  }

  /**
   * Get the field value.
   *
   * @param object $obj
   *   The entity object.
   * @param string $field
   *   The field name.
   * @param mixed $default
   *   Teh default value.
   *
   * @return mixed
   *   The field value.
   */
  public function getVal($obj, $field, $default = '') {
    return $this->hasField($obj, $field) ? $obj->get($field)->value : $default;
  }

  /**
   * Get the view IDs.
   *
   * @return array
   *   An array of view IDs.
   */
  public function getViewIds() {
    return !empty($this->getCfg('view_ids')) ? (array) $this->getCfg('view_ids') : [];
  }

  /**
   * Determine if the object has the field and that it's non-empty.
   *
   * @param object $obj
   *   The entity object.
   * @param string $field
   *   The field name.
   *
   * @return bool
   *   True if the object has the field and that it's non-empty.
   */
  public function hasField($obj, $field) {
    return (!empty($obj) && !empty($field) && $obj->hasField($field) && !$obj->get($field)->isEmpty()) ? TRUE : FALSE;
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   * @param mixed $val
   *   The field value.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  public function setCfg($key = '', $val = NULL) {
    $id = $this->getModuleName() . '.config';

    $cfg = $this->cfg->getEditable($id);

    return $key ? $cfg->set($key, $val)->save() : NULL;
  }

  /**
   * Redirect the bot if we have an error code.
   */
  public function redirectBadBot($tgt = '/500.html') {
    if (!$this->getErrorStatus($this->getIp())) {
      return;
    }
    $response = new RedirectResponse($tgt);
    $response->send();
  }

  /**
   * Insert an IP with warning status.
   *
   * @param string $ip
   *   The IP.
   * @param mixed $data
   *   The data dump.
   *
   * @return object
   *   A new InsertQuery object for this connection.
   */
  public function warnIp($ip, $data) {
    $fields = [
      'timestamp' => time(),
      'ip'        => $ip,
      'level'     => static::STATUS_WARN,
      'data'      => print_r($data, TRUE),
    ];
    return $this->sql->insertIp($fields);
  }

}
