<?php

namespace Drupal\botlog;

use Drupal\Core\Database\Connection;

/**
 * SQL for the botlog module.
 */
class BotlogSql {

  /**
   * Database connection.
   *
   * @var Drupal\Core\Database\Connection
   */
  protected $dbh;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Database\Connection $dbh
   *   The database handle.
   */
  public function __construct(Connection $dbh) {
    $this->dbh = $dbh;
  }

  /**
   * Delete all entries with the supplied IP.
   *
   * @param int $ip
   *   The ip.
   *
   * @return int
   *   The number of deleted entries.
   */
  public function deleteByIp($ip) {
    return $this->dbh->delete('botlog')->condition('ip', $ip)->execute();
  }

  /**
   * Delete old entries.
   *
   * @param int $timestamp
   *   The max time of entries to delete.
   *
   * @return int
   *   The number of deleted entries.
   */
  public function deleteByTime($timestamp) {
    return $this->dbh->delete('botlog')
      ->condition('timestamp', $timestamp, '<=')
      ->condition('level', 2, '<=')
      ->execute();
  }

  /**
   * Get the offer's expired status.
   *
   * @param int $ip
   *   The IP.
   * @param int $status
   *   The status.
   *
   * @return int
   *   One if IP has an error set.
   */
  public function getErrorStatus($ip, $status) {
    $stmt = "SELECT 1 FROM {botlog} WHERE ip = :ip AND level = :level";
    $vals = [
      ':ip'    => $ip,
      ':level' => $status,
    ];
    return $this->dbh->queryRange($stmt, 0, 1, $vals)->fetchField();
  }

  /**
   * Fetch the row count.
   *
   * @return int
   *   The number of rows in the botlog table for the IP.
   */
  public function getCount() {
    return $this->dbh->query("SELECT COUNT(*) FROM {botlog} ")->fetchField();
  }

  /**
   * Fetch the row count.
   *
   * @param int $ip
   *   The IP.
   *
   * @return int
   *   The number of rows in the botlog table for the IP.
   */
  public function getIpCount($ip) {
    return $this->dbh->query("SELECT COUNT(*) FROM {botlog} WHERE ip = :ip", [':ip' => $ip])->fetchField();
  }

  /**
   * Insert a monotoring entry.
   *
   * @param array $fields
   *   An array of the monotoring fields.
   *
   * @return object
   *   A new InsertQuery object for this connection.
   */
  public function insertIp(array $fields) {
    return $this->dbh->insert('botlog')->fields($fields)->execute();
  }

  /**
   * Fetch all the events.
   *
   * @param int $start
   *   The starting offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   An array of event objects.
   */
  public function selectAllEvents($start = NULL, $limit = 100) {
    if (is_numeric($start) && intval($limit)) {
      return $this->dbh->queryRange("SELECT * FROM {botlog} ORDER BY timestamp DESC", $start, $limit)->fetchAll();
    }
    return $this->dbh->query("SELECT * FROM {botlog} ORDER BY timestamp DESC")->fetchAll();
  }

  /**
   * Fetch the error.
   *
   * @param int $bid
   *   The error ID.
   *
   * @return object
   *   The error row.
   */
  public function selectEvent($bid) {
    return $this->dbh->query("SELECT * FROM {botlog} WHERE bid = :bid", [':bid' => $bid])->fetchObject();
  }

}
